/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "netlistviewitem.h"

#include <kiconloader.h>
#include <kiconeffect.h>

void NetListViewItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment)
{

        if (column == mQualityColumn) {
                KListViewItem::paintCell(p, cg, column, width, alignment);
                int barWidth;
                barWidth = mQuality;
                while (barWidth % 8 != 0)
                        barWidth=barWidth-1;
                barWidth+=8;

                int top = int((this->height()-16)/2);
                QPixmap qualityIcon = SmallIcon("knewstuff");
                QPixmap qualityIconGray = KIconEffect().apply( qualityIcon, KIconEffect::ToGray, 1, Qt::white, true );
                p->drawTiledPixmap(listView()->itemMargin(),top,6*16, 16, qualityIconGray ); //6*16 -> width of six icon (best quality)
                p->drawTiledPixmap(listView()->itemMargin(),top,barWidth, 16, qualityIcon );
        } else if ((column == mEncColumn) && (mEnc)) {
                KListViewItem::paintCell(p, cg, column, width, alignment);
                this->setPixmap(mEncColumn, SmallIcon("encrypted") );
        } else if (column == mEssidColumn) {
                QColorGroup mCg = cg;
                if (mQuality < 8)  // gray-out essid names of networks with very low link quality
                        mCg.setColor( QColorGroup::Text, QColor(120,120,120) );
                this->setText(mEssidColumn, mEssid);
                if (mEssid=="<hidden>") { //write '<hidden>' in italics.
                	QFont mFont = listView()->font();
                	mFont.setItalic(1);
                	p->setFont( mFont );
                }
                KListViewItem::paintCell(p, mCg, column, width, alignment);
        } else if (column == mChanColumn) {
                QColorGroup mCg = cg;
                mCg.setColor( QColorGroup::Text, QColor(120,120,120) );
                QFont mFont = listView()->font();
                mFont.setItalic(1);
                p->setFont( mFont );
                this->setText(mChanColumn, mChannel);
                KListViewItem::paintCell(p, mCg, column, width, alignment);
        /*} else if (column == mModeColumn) {
                QColorGroup mCg = cg;
                mCg.setColor( QColorGroup::Text, QColor(140,140,140) );
                QFont mFont = listView()->font();
                mFont.setItalic(1);
                p->setFont( mFont );
                this->setText(mModeColumn, mMode); //.upper() );
                KListViewItem::paintCell(p, mCg, column, width, alignment);*/
        } else if (column == mAPColumn) {
                QColorGroup mCg = cg;
                if (mQuality < 8)  // gray-out essid names of networks with very low link quality
                        mCg.setColor( QColorGroup::Text, QColor(120,120,120) );
                this->setText(mAPColumn, mAP);
                KListViewItem::paintCell(p, mCg, column, width, alignment);
        }
        else
                KListViewItem::paintCell(p, cg, column, width, alignment);
}

int NetListViewItem::width(const QFontMetrics &fm, const QListView *lv, int column) const
{
        int w;
        if (column == mQualityColumn)
                w = 6*16 + (lv->itemMargin()*2);
        else if (column == mEncColumn)
                w = 16 + (lv->itemMargin()*2);
        else if (column == mChanColumn)
                w = fm.width( mChannel ) + (lv->itemMargin()*2);
        /*else if (column == mModeColumn)
                w = fm.width( mMode ) + (lv->itemMargin()*2);*/
        else if (column == mEssidColumn)
                w = fm.width( mEssid ) + (lv->itemMargin()*2) + lv->lineWidth();
        else if (column == mAPColumn)
                w = fm.width( mAP ) + (lv->itemMargin()*2) + lv->lineWidth();
        else
                w = 0;

        int headerw = fm.width( listView()->columnText(column) ) + (lv->itemMargin()*2);
        if (w < headerw)
                w = headerw;

        return w;
}

QString NetListViewItem::key( int column, bool ascending ) const
{

        QString t = QString::null;
        if (column == mQualityColumn) {
                t = QString::number( mQuality );
                if (mQuality < 10)
                        t.prepend("0");
        } else if (column == mEncColumn) {
                if (mEnc)
                        t = "1";
                else
                        t="0";
        } else if (column == mEssidColumn) {
                t = mEssid.upper(); // add .upper() to make it case-insensitive;
        } else if (column == mChanColumn) {
                t = mChannel;
        /*} else if (column == mModeColumn) {
                t = mMode.upper();*/
        } else if (column == mAPColumn) {
                t = mAP.upper();
        }


        return t;
}
