/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "wlassistant.h"
#include "netlistviewitem.h"
#include "waconfig.h"
#include "ui_confgeneral.h"
#include "ui_netparamswizard.h"

//#include <unistd.h>
#include <iostream>

#include <qlabel.h>
#include <qlistview.h>
#include <qprocess.h>
#include <qcursor.h>
#include <qlistbox.h>
#include <qprogressbar.h>
#include <qpainter.h>
#include <qeventloop.h>
#include <qtimer.h>

#include <qpopupmenu.h>
#include <qcursor.h>

#include <kcombobox.h>
#include <klistview.h>
#include <kapplication.h>
#include <kconfigdialog.h>
#include <kiconloader.h>
#include <kurlrequester.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kpopupmenu.h>

WirelessAssistant::WirelessAssistant(QWidget* parent, const char* name, bool modal, WFlags fl)
                : mainWindow(parent,name, modal,fl)
{
        buttonScan->setIconSet( SmallIcon("find") );
        buttonConnect->setIconSet( SmallIcon("connect_creating") );
        buttonSettings->setIconSet( SmallIcon("configure") );
        buttonClose->setIconSet( SmallIcon("fileclose") );
        //buttonStop->setIconSet( SmallIcon("stop") );
        //buttonStop->hide(); ////////////// REMOVE WHEN REIMPLEMENTING THE IDEA.

        netList->setAllColumnsShowFocus(1);
        netList->setColumnAlignment(1, Qt::AlignRight );
        netList->setColumnAlignment(2, Qt::AlignRight );
        netList->setAlternateBackground( QColor(238, 246, 255) );
        netList->setSorting( 0, 1 );
        netList->setItemMargin(8);
        frameDevice->hide();

        connect( devCombo, SIGNAL(activated( const QString & )),
                 this, SLOT(setDev( const QString & ) ));

        QTimer::singleShot(10, this, SLOT(init()) ); //WAIT FOR THE UI TO BE READY BEFORE FURTHER SETUP (msec)
}

WirelessAssistant::~WirelessAssistant()
{}

/*$SPECIALIZATION$*/

void WirelessAssistant::init()
{
        ////////////////////////////////////////
        ///// CHECK FOR SYSFS (KERNEL 2.6) /////
        if ( !QFile::exists("/sys") ) {
                std::cout << "Sysfs not present." << std::endl;
                KMessageBox::error(0, "Kernel 2.6 or later not present.\nWireless Assistant will now quit.");
                close();
                return;
        }

        ///////////////////////////////////
        ///// DETECT WIRELESS DEVICES /////
        statusLabel->setText("Detecting wireless devices...");
        QString devPath = "/sys/class/net/";
        QDir *devDir = new QDir( devPath );
        devDir->setFilter( QDir::Dirs | QDir::Readable );
        QStringList devList = devDir->entryList();
        delete devDir;

        std::cout << "Wireless interface(s): ";
        for ( QStringList::Iterator dev = devList.begin(); dev != devList.end(); dev++ ) {
                if ( *dev == ".." || *dev == "lo" )
                        continue;
                if ( ((*dev).left(4) == "wlan" || (*dev).left(3) == "ath") || QDir( QString(devPath+(*dev)+"/wireless") ).exists() ) {
                        std::cout << (*dev);
                        devCombo->insertItem(*dev);
                }
        }

        statusLabel->setText("Finished detecting devices.");
        if (devCombo->count() == 0) {
                std::cout << "none" << std::endl;
                KMessageBox::error(0, "No usable wireless device found.\nWireless Assistant will now quit.");
                close();
                return;
        }
        std::cout << std::endl;
        if (devCombo->count() > 1)
                frameDevice->show();
        NetParams.iface = devCombo->currentText();	// set interface name

        //////////////////////////////////
        ///// CHECK FILE PERMISSIONS /////
        /// @todo Permission Checking (Set or Warn&Continue)
        if (!QFileInfo("/etc/resolv.conf").isWritable()) {
                std::cout << "warning: /etc/resolv.conf not writable" << std::endl;
                KMessageBox::information(0, "You might have insufficient permissions for Wireless Assistant to function properly.\nDid you run it using 'sudo'?");
        }

        ///////////////////////////////////////
        ///// UPDATE CONFIG TO UPDATE GUI /////
        updateConfiguration();

        ////////////////////////
        ///// DETECT PATHS /////
        Commands.init();
        if (!Commands.allFound) {	//all ok or ONLY dhcpcd not found (i.e. dhclient present).
                KMessageBox::error(0, "Executable(s) '"+Commands.notFound.join("', '")+"' could not be found.\nWireless Assistant will now quit.");
                close();
                return;
        }

        /////////////////////////////////////////////////////
        ///// LOAD CONFIG FILE INCL. ALL NET PARAMETERS /////
        WaSettings::self()->addItemStringList("NetParamsList", NetParamsList);
        WaSettings::self()->readConfig();
        //removeNetParams();
        //KMessageBox::error(0, NetParamsList.join("\n"));

        ///////////////////////////////////////
        ///// SCAN FOR AVAILABLE NETWORKS /////
        netScan( NetParams );
}

void WirelessAssistant::removeNetParams()
{
        QString ap = static_cast<NetListViewItem*>(netList->selectedItem())->ap();
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        if ( KMessageBox::warningContinueCancel(0, QString("Settings for network '"+matchEssidForAp(ap)+"' are about to be deleted.\nDo you want to continue?")) == 5 ) //YES
                        {
                                NetParamsList.remove(nps);
                                WaSettings::writeConfig();
                        }
                        break;
                }
        }
}


void WirelessAssistant::setDNS( const WANetParams & np )
{
        QFile f("/etc/resolv.conf");
        if (f.open( IO_WriteOnly | IO_Truncate )) {
                QTextStream s( &f );
                if (!np.domain.isEmpty()) {
                        s << QString("domain " + np.domain + "\n");
                        std::cout << "resolv.conf: domain " << np.domain << std::endl;
                }
                if (!np.dns1.isEmpty()) {
                        s << QString("nameserver " + np.dns1 + "\n");
                        std::cout << "resolv.conf: nameserver " << np.dns1 << std::endl;
                }
                if (!np.dns2.isEmpty()) {
                        s << QString("nameserver " + np.dns2 + "\n");
                        std::cout << "resolv.conf: nameserver " << np.dns2 << std::endl;
                }
                f.close();
        } else {
                std::cout << "dns setup error: " << f.name() << " is not writeable." << std::endl;
                KMessageBox::error(0, QString("File '"+f.name()+"' could not be opened for writing.\nNameserver(s) and/or domain are not set."));
        }
}

void WirelessAssistant::netScan()
{
        netScan( NetParams );
}

void WirelessAssistant::netScan( const WANetParams & np )
{
        QString result;

        setUi(0);
        netList->clear();
        statusLabel->setText("Bringing interface up...");
        result = runCommand( Commands.cmd("ifup",np) );
        if ( result == "ERR" || result == "TERM" )
                return;

        statusLabel->setText("Scanning...");
        result = runCommand( Commands.cmd("scan",np) );

        if ( result == "ERR" || result == "TERM" )
                return;
        else
                parseScan( result );
}

void WirelessAssistant::parseScan( QString output )
{
        QString essid;
        QString channel;
        QString mode;
        int qualInt;
        bool enc;
        QString ap;

        bool ok_channel = 1;
        bool ok_noise = 1;//does iwlist return channel/noise?

        for (int i=0; (!output.section("Cell ",i,i).isEmpty()); i++ ) {
                // GET ESSID VALUE
                essid = getVal(output, "ESSID:\"","\"","Cell ",i);

                // GET CHANNEL NUMBER
                channel = getVal(output, "Channel:"," ","Cell ",i);
                if (channel.isEmpty()) {
                        channel = getVal(output, "Frequency:","Hz","Cell ",i);
                        ok_channel = 0;
                }

                // GET MODE VALUE
                /*mode = getVal(output, "Mode:"," ","Cell ",i);

                if (mode.upper()=="AD-HOC")
                        continue; // Ad-Hoc connections not supported.

                if (mode.upper()=="MASTER")
                        mode="Managed"; //Some drivers (madwifi...) report Managed networks as Master. Workaround.*/

                // GET AP
                ap = getVal(output, "Address: "," ","Cell ",i);

                if (essid.isEmpty()) {
                        if (!ap.isEmpty())
                                essid = matchEssidForAp( ap );
                        else
                                continue;	//some cards report one '' essid even when no network's present. Workaround.
                }

                // GET QUALITY
                int wnoise, wsignal;
                //check if driver reports quality directly
                qualInt = getVal(output, "Quality","/","Cell ",i,1).toInt();

                if (qualInt == 0) { //not directly reported? What a pity.
                        wnoise = getVal(output, "Noise level","dBm","Cell ",i,2).toInt();
                        wsignal = getVal(output, "Signal level","dBm","Cell ",i,2).toInt();
                        if (wnoise!=0)
                                qualInt = int( (wnoise-wsignal)*100 /wnoise ); //noise reported? cool.
                        else {		//noise not detected? do some estimation.
                                qualInt = 100-wsignal;
                                if (qualInt <1)
                                        qualInt=1;
                                ok_noise = 0;
                        }
                }
                if (qualInt>94)
                        qualInt=94;


                // GET ENCRYPTION
                if (getVal(output, "Encryption key:"," ","Cell ",i).upper()=="ON")
                        enc = 1;
                else
                        enc = 0;

                new NetListViewItem( netList, essid, channel, qualInt, enc, ap );
        }

        if (!ok_channel)
                netList->setColumnText( 1, "Freq (Hz)" );
        if (!ok_noise)
                netList->setColumnText( 2, "Signal level" );
        uiScanned();
}

void WirelessAssistant::uiScanned()
{
        setNetListColumns();

        if (netList->childCount() > 0) {
                buttonScan->setText("Re&scan");
                statusLabel->setText( "Done." );
        } else {
                statusLabel->setText("No wireless networks found!");
        }
        setUi(1);
}

void WirelessAssistant::setNetParamsFromList( QListViewItem* lvi )
{
        NetListViewItem *nvi = static_cast<NetListViewItem*>(lvi);
        NetParams.essid = nvi->essid();
        if (NetParams.essid == "<hidden>")
                NetParams.hiddenEssid = 1;
        else
                NetParams.hiddenEssid = 0;
        //NetParams.mode = nvi->mode();
        NetParams.channel = nvi->channel();
        NetParams.ap = nvi->ap();
        NetParams.wep = nvi->enc();
}

bool WirelessAssistant::setNetParamsFromConfig( const QString & ap )
{
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        NetParams.loadNetParamsString( *nps );
                        return 1;
                }
        }
        return 0;
}

void WirelessAssistant::netConnect()
{
        if (!netList->selectedItem())
                return;
        setNetParamsFromList( netList->selectedItem() );

        if ( (NetParams.essid=="<hidden>") || (!setNetParamsFromConfig( NetParams.ap )) ) {
                ui_NetParamsWizard *netwiz = new ui_NetParamsWizard;
                if (!NetParams.hiddenEssid)
                        netwiz->setCaption(QString(NetParams.essid+" - First Connection Wizard"));
                netwiz->setEssidEnabled( NetParams.hiddenEssid );
                netwiz->setWepEnabled( NetParams.wep );
                netwiz->exec();
                if (netwiz->result()==QDialog::Rejected) {
                        delete netwiz;
                        return;
                } else {
                        NetParams = netwiz->readNetParams( NetParams );
                        /// @todo Check if WEP/Channel/AP settings haven't changed. Ask to review settings if necessary.
                        NetParamsList << NetParams.netParamsString();
                        static_cast<NetListViewItem*>(netList->selectedItem())->setEssid( NetParams.essid );
                        WaSettings::self()->writeConfig();
                        delete netwiz;
                }
        }
        netConnect( NetParams );
}

QString WirelessAssistant::matchEssidForAp( const QString & ap )
{
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        return (*nps).section(",",1,1); //essid
                }
        }
        return "<hidden>";
}

void WirelessAssistant::netConnect( const WANetParams & np )
{
        statusLabel->setText( QString("Connecting to '"+np.essid+"'...") );
        setUi(0);

        runCommand( Commands.cmd("ifdown", np) );
        runCommand( Commands.cmd("iwconfig", np) );
        runCommand( Commands.cmd("iwconfig_ap", np) );
        runCommand( Commands.cmd("kill_dhcp", np) );
        if (np.dhcp) //DHCP config
                runCommand( Commands.cmd("ifconfig_dhcp", np) );
        else { //manual config
                runCommand( Commands.cmd("ifconfig_manual", np) );
                setDNS( np );
                runCommand( Commands.cmd("route", np) );
        }
        //////////////////////
        ///// CHECK CONNECTION
        statusLabel->setText("Testing connection...");
        statusLabel->repaint();

        if (isConnected()) {
                if (WaSettings::autoQuit())
                        this->close();
                statusLabel->setText( QString("Successfully connected to '"+np.essid+"'.") );
        } else
                statusLabel->setText("Connection failed.");
        setUi(1);
}

void WirelessAssistant::showCfgDlg()
{

        if ( KConfigDialog::showDialog( "conf" ) )
                return;

        KConfigDialog* dialog = new KConfigDialog( this, "conf", WaSettings::self() );

        ui_confGeneral* confG = new ui_confGeneral();
        dialog->addPage( confG, "General", "kcontrol" );

        connect( dialog, SIGNAL(settingsChanged()),
                 this, SLOT(updateConfiguration()) );

        dialog->show();
}

void WirelessAssistant::updateConfiguration()
{
        if (WaSettings::singleClick()) {
                setSingleClick( 1 );
                buttonConnect->hide();
        } else {
                setSingleClick( 0 );
                buttonConnect->show();
        }
}

void WirelessAssistant::setSingleClick(bool i)
{
        if (i)
                connect( netList, SIGNAL(clicked(QListViewItem*)),
                         this, SLOT(netConnect()) );
        else
                disconnect( netList, SIGNAL(clicked(QListViewItem*)),
                            this, SLOT(netConnect()) );

}

void WirelessAssistant::setDev( const QString & ifname)
{
        NetParams.iface = ifname;
        std::cout << "Selected interface: " << ifname << std::endl;
}

/*QString WirelessAssistant::getIfName(QString ifName)
{
        QString devId,devName,execpath;
        int marker,len;
 
        // get device id
        execpath=getPath("udevinfo");
        if (execpath.isEmpty())
                return QString::null;
        QProcess* proc = new QProcess( this );
        proc->addArgument(execpath);
        proc->addArgument("-ap");
        proc->addArgument( QString("/class/net/"+ifName) );
        if (!proc->start())
                return QString::null;
while (proc->isRunning()) {}
        devId = QString(proc->readStdout());
        marker=devId.find("\"device\"");
        marker=devId.find("ID=",marker);
        marker=devId.find(":",marker)+1;
        len = devId.find("\"",marker)-marker;
        devId=devId.mid(marker, len);
 
        //get human-readable interface name
        execpath=getPath("lspci");
        if (execpath.isEmpty())
                return QString::null;
        proc = new QProcess( this );
        proc->addArgument(execpath);
        if (!proc->start())
                return QString::null;
while (proc->isRunning()) {}
        devName = QString(proc->readStdout());
        marker=devName.find(devId)+devId.length();
        marker=devName.find(":",marker)+2;
        len=devName.find("(",marker)-1-marker;
        devName=devName.mid(marker,len);
 
        delete proc;
        return devName;
}*/

QString WirelessAssistant::runCommand( QStringList cmd )
{
        if (cmd.isEmpty())
                return QString::null;

        QProcess* p = new QProcess( this );
        p->setArguments( cmd );
        p->start();
        while ( p->isRunning() ) { // PROCESS USER EVENTS
                KApplication::eventLoop()->processEvents( QEventLoop::AllEvents );
        }
        QString e = QString( p->readStderr() );
        if (!e.isEmpty())
                std::cout << "==>stderr: " << e;// << std::endl;
        return QString( p->readStdout() );
}

void WirelessAssistant::setUi(int uiState)
{

        if (uiState==0) {
                devCombo->setEnabled( false );
                buttonScan->setEnabled( false );
                buttonConnect->setEnabled( false );
                buttonSettings->setEnabled( false );
                //buttonDetect->setEnabled( false );
                setSingleClick(0);
                KApplication::setOverrideCursor( QCursor(Qt::BusyCursor) );
                disconnect( netList, SIGNAL(rightButtonPressed( QListViewItem*, const QPoint&, int )),
                            this, SLOT(showItemContextMenu( QListViewItem*, const QPoint&, int )) );
        } else {
                if (devCombo->count() > 0) {
                        devCombo->setEnabled( true );
                        buttonScan->setEnabled( true );
                }
                if (netList->childCount() > 0)
                        buttonConnect->setEnabled( true );
                buttonSettings->setEnabled( true );
                //buttonDetect->setEnabled( true );
                if (WaSettings::singleClick())
                        setSingleClick(1);
                KApplication::restoreOverrideCursor();
                connect( netList, SIGNAL(rightButtonPressed( QListViewItem*, const QPoint&, int )),
                         SLOT(showItemContextMenu( QListViewItem*, const QPoint&, int )) );

        }
}

void WirelessAssistant::showItemContextMenu( QListViewItem* i, const QPoint& p, int c )
{
        if (!i)
                return;

        QString ap = static_cast<NetListViewItem*>(i)->ap();
        bool isConfigured = setNetParamsFromConfig(ap);
        KPopupMenu *icm = new KPopupMenu();
        icm->insertTitle(static_cast<NetListViewItem*>(i)->essid());
        if (isConfigured) {
                icm->insertItem( SmallIcon("connect_creating"), "Connect", this, SLOT(netConnect()) )
                ;
                icm->insertSeparator();
                icm->insertItem("Forget Settings...", this, SLOT(removeNetParams()) );
                icm->insertItem("Edit Settings...", this, SLOT(editNetParams()) );
        } else {
                icm->insertItem( SmallIcon("connect_creating"), "Configure and Connect...", this, SLOT(netConnect()) );
        }
        icm->exec( QCursor::pos() );
}

void WirelessAssistant::editNetParams()
{
        setNetParamsFromList( netList->selectedItem() );
        KMessageBox::sorry(0, "Editing per network settings is not yet implemented.\nTry selecting 'Forget Settings...' and entering new values using the wizard.");
}

void WirelessAssistant::setNetListColumns()
{
        int realWidth = netList->viewportSize( netList->contentsWidth(), netList->contentsHeight() ).width(); //calculate actual width taking scrollbars into account
        int essidWidth = realWidth - netList->columnWidth(1) - netList->columnWidth(2) - netList->columnWidth(3);

        netList->setColumnWidth(0, essidWidth);
}

bool WirelessAssistant::isConnected()
{
        QString result;
        if (!NetParams.dhcp) {
                result = runCommand( Commands.cmd( "ping_gw", NetParams ) );
                if ( (result.isEmpty()) || (result.find("100%") > -1) ) // "Network Unreachable" OR "100% packet loss (only 1 sent, 0 received)
                {
                        std::cout << "==>error: could not ping gateway (" << NetParams.gateway << ")\n";
                        std::cout << "CONNECTION FAILED" << std::endl;
                        return 0;
                } else
                        std::cout << "Gateway successfully reached." << std::endl;
        }

        result = runCommand( Commands.cmd( "ping_internet", NetParams ) );
        if ( (result.isEmpty()) || (result.find("100%") > -1) ) {
                std::cout << "==>error: could not ping internet\n";
                std::cout << "CONNECTION FAILED" << std::endl;
                return 0;
        } else {
                std::cout << "Internet successfully reached\nCONNECTION OK" << std::endl;
                return 1;
        }
}

QString WirelessAssistant::getVal(QString str, QString start, QString end, QString sectDelimit, int sectNo, int offset)
{
        int mark1,mark2;

        if (!sectDelimit.isEmpty())
                str = str.section(sectDelimit, sectNo, sectNo);
        if (str.isEmpty())
                return QString::null;

        str.simplifyWhiteSpace();

        if (start.isEmpty())
                mark1=0;
        else {
                mark1=str.find(start);
                if (mark1==-1)
                        return QString::null;
                mark1+= start.length();
        }


        if (end.isEmpty())
                mark2=str.length();
        else
                mark2=str.find(end,mark1);

        if (mark2==-1)
                return QString::null;

        mark1 += offset; //add offset to the beginning
        return str.mid(mark1,mark2-mark1).stripWhiteSpace();
}


#include "wlassistant.moc"
